<?php
// $Id: search_api_facets.theme.inc,v 1.1.2.3 2010/11/30 16:39:48 drunkenmonkey Exp $

/**
 * Theme a list of facet links.
 *
 * @param array $variables
 *   An associative array containing:
 *   - terms: An array of available facet terms, as render arrays (or HTML).
 *   - active: An array of active facet filters, as render arrays (or HTML).
 */
function theme_search_api_facets_list(array $variables) {
  $items = array();
  foreach ($variables['active'] as $filter) {
    $items[] = render($filter);
  }
  foreach ($variables['terms'] as $term) {
    $items[] = render($term);
  }
  return $items ? theme('item_list', array('items' => $items, 'class' => array('search-api-facets'))) : '';
}

/**
 * Theme a single facet item.
 *
 * @param array $variables
 *   An associative array containing:
 *   - name: The name to display for the item.
 *   - count: The number of results for a non-active facet.
 *   - path: The destination path when the facet link is clicked.
 *   - options: An array of options to pass to l().
 *   - active: A boolean telling whether this facet filter is active or not.
 */
function theme_search_api_facets_facet(array $variables) {
  $name = $variables['name'];
  $path = $variables['path'];
  $options = $variables['options'];

  if ($variables['active']) {
    return l('(-)', $path, $options) . ' ' . check_plain($name);
  }
  return l($name . ' (' . $variables['count'] . ')', $path, $options);
}

/**
 * Theme one item of the "Current search" block.
 *
 * @param array $variables
 *   An associative array containing:
 *   - index: The SearchApiIndex the search was executed on.
 *   - keys: The fulltext keys used.
 *   - count: The number of results found.
 *   - path: The destination path when the facet link is clicked.
 *   - options: An array of options to pass to l().
 *   - filters: An array of active facet filters for this search, as render
 *     arrays (or HTML).
 */
function theme_search_api_facets_current_search(array $variables) {
  $items = array();
  foreach ($variables['filters'] as $filter) {
    $items[] = render($filter);
  }
  $items = $items ? theme('item_list', array('items' => $items, 'class' => array('search-api-facets'))) : '';
  $link = l($variables['keys'], $variables['path'], $variables['options']);
  $results = format_plural($variables['count'], '1 result', '@count results');
  return '<h3>' . $link . ' (' . $results . ")</h3>\n" . $items;
}
